import log from 'loglevel'

export default createLoggerMiddleware

/**
 * Returns a middleware that logs RPC activity
 * @param {{ origin: string }} opts - The middleware options
 * @returns {Function}
 */
function createLoggerMiddleware(opts) {
  return function loggerMiddleware(
    /** @type {any} */ req,
    /** @type {any} */ res,
    /** @type {Function} */ next
  ) {
    next((/** @type {Function} */ cb) => {
      if (res.error) {
        log.error('Error in RPC response:\n', {
          ...res,
          method: req.method,
          params: req.params,
        })
      }
      if (req.isMetamaskInternal) {
        return
      }
      log.info(`RPC (${opts.origin}):`, req, '->', {
        ...res,
        method: req.method,
        params: req.params,
      })
      cb()
    })
  }
}
